#include "StdAfx.h"
#include "LineList.h"
#include "Registry.h"
#include "ServerSIP.h"

cLineData::cLineData()
{
	m_sCodecList = gcnew String("");
	m_sLineName = gcnew String("");
	
	m_nLineId = 0;
	m_nCallType = -1;
	m_bRegister = false;
}

cLineList::cLineList(void)
{
	m_nLineIdCount = 0;
	
	m_objLineListTEXT = gcnew Hashtable();
    m_objLineListINT = gcnew Hashtable();
}

int cLineList::ChooseLineId()
{
    while (true)
    {
        m_nLineIdCount++;
        if (FindLine(m_nLineIdCount) == nullptr) break;
    }

    return m_nLineIdCount;
}

void cLineList::UnLoadLines()
{
    ArrayList^ objCloseLineList = gcnew ArrayList();
    IDictionaryEnumerator^ enumerator = m_objLineListTEXT->GetEnumerator();

    while(enumerator->MoveNext())
        objCloseLineList->Add(enumerator->Key);

    for(int nCount = 0; nCount < objCloseLineList->Count; nCount++)
        Remove((String^) objCloseLineList[nCount]);
    
    m_objLineListTEXT->Clear();
    m_objLineListINT->Clear();

}

void cLineList::LoadLines()
{
    UnLoadLines();

    ArrayList^ objLineList = gcnew ArrayList();
    cRegistry^ objReg = gcnew cRegistry();

    objReg->EnumLine(objLineList);
    cLineReg^ objLineReg = gcnew cLineReg();

    for(int nCount = 0; nCount < objLineList->Count; nCount++)
    {
        String^ sLineName = (String^) objLineList[nCount];

        if(objReg->GetLine(sLineName, objLineReg) == false)
        {
            cServerSIP::m_objServerSIP->LogError("Unable to extract line information from registry.");
            continue;
        }

        Add(sLineName, objLineReg->m_sDisplayName, objLineReg->m_sUserName, objLineReg->m_sAuthUser, objLineReg->m_sPassword, objLineReg->m_sRealm, objLineReg->m_sProxy, objLineReg->m_sOutboundProxy, objLineReg->m_sCodec, objLineReg->m_bRegister, objLineReg->m_sInCallType, objLineReg->m_sInCallTo);
    }
}

bool cLineList::AddLine(String^ sLineName, String^ sDisplayName, String^ sUserName, String^ sAuthUser, String^ sAuthPwd, String^ sRealm, String^ sProxy, String^ sOutboundProxy, String^ sCodec, Boolean bRegister, String^ sInCallType, String^ sInCallTo)
{
    cLineReg^ objLineReg = gcnew cLineReg();

    objLineReg->m_sDisplayName = sDisplayName;
    objLineReg->m_sUserName = sUserName;
    objLineReg->m_sAuthUser = sAuthUser;
    objLineReg->m_sPassword = sAuthPwd;
    objLineReg->m_sRealm = sRealm;
    objLineReg->m_sProxy = sProxy;
    objLineReg->m_sOutboundProxy = sOutboundProxy;
    objLineReg->m_sCodec = sCodec;

    objLineReg->m_bRegister = bRegister;

    objLineReg->m_sInCallType = sInCallType;
    objLineReg->m_sInCallTo = sInCallTo;

    cRegistry^ objReg = gcnew cRegistry();
    if(objReg->SetLine(sLineName, objLineReg) == false)
        return false;

    Add(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sRealm, sProxy, sOutboundProxy, sCodec, bRegister, sInCallType, sInCallTo);
    
    return true;
}

bool cLineList::RemoveLine(String^ sLineName)
{
    cRegistry^ objReg = gcnew cRegistry();

    if(objReg->RemoveLine(sLineName) == false)
        return false;

    Remove(sLineName);
    return true;
}

void cLineList::Add(String^ sLineName, String^ sDisplayName, String^ sUserName, String^ sAuthUser, String^ sAuthPwd, String^ sRealm, String^ sProxy, String^ sOutboundProxy, String^ sCodec, Boolean bRegister, String^ sInCallType, String^ sInCallTo)
{
    Remove(sLineName);

    cLineData^ objLineData = gcnew cLineData();

    objLineData->m_nCallType = Convert::ToInt32(sInCallType);
    objLineData->m_sCallTo = sInCallTo;
    objLineData->m_bRegister = bRegister;

    objLineData->m_sLineName = sLineName;
    objLineData->m_sCodecList = sCodec;

    int nLineId = ChooseLineId();
    objLineData->m_nLineId = nLineId;

    m_objLineListINT->Add(nLineId, objLineData);

    sLineName = sLineName->ToLower();
    m_objLineListTEXT->Add(sLineName, objLineData);

    if(cServerSIP::m_objServerSIP->AddLine(nLineId, sUserName, sAuthUser, sAuthPwd, sDisplayName, sRealm, sProxy, sOutboundProxy) == false)
    {
        cServerSIP::m_objServerSIP->LogError("");
        return;
    }

    if(bRegister)
    {
        if(cServerSIP::m_objServerSIP->RegisterLine(nLineId) == false)
            cServerSIP::m_objServerSIP->LogError("");
    }
}

void cLineList::Remove(String^ sLineName)
{
    sLineName = sLineName->ToLower();

    cLineData^ objLineData = FindLine(sLineName);
    if(objLineData == nullptr) return;

    m_objLineListINT->Remove(objLineData->m_nLineId);
    m_objLineListTEXT->Remove(sLineName);

    if(objLineData->m_bRegister)
        cServerSIP::m_objServerSIP->UnRegisterLine(objLineData->m_nLineId);
    
    objLineData = nullptr;
}

cLineData^ cLineList::FindLine(int nLineId)
{
    return (cLineData^) m_objLineListINT[nLineId];
}

cLineData^ cLineList::FindLine(String^ sLineName)
{
    return (cLineData^) m_objLineListTEXT[sLineName->ToLower()];
}
