using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace VaxTeleConfigSIP
{
    public partial class AddLineForm : Form
    {
        public const int IN_BOUND_CALL_REJECT = 0;
        public const int IN_BOUND_CALL_CONNECT_USER = 1;
        public const int IN_BOUND_CALL_PUT_IN_QUEUE = 2;

        public AddLineForm()
        {
            InitializeComponent();
        }

        private Boolean AddLineChecks(ref string sInCallTo, ref string sInCallType)
        {
            if(ComboLine.Text.Length == 0)
            {
                MessageBox.Show("Please enter Line Name.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ListBoxEnableCodec.Items.Count == 0)
            {
                MessageBox.Show("Please select a codec.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            sInCallType = "0";

            if(RadioConnectToUser.Checked == true)
            {
                if(ComboConnectToUser.SelectedIndex == -1)
                {
                    MessageBox.Show("Please select the user/login to forward the incoming call.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }

                sInCallType = "1";
                sInCallTo = ComboConnectToUser.SelectedItem.ToString();
            }
            else if(RadioPutInQueue.Checked == true)
            {
                if(ComboPutInQueue.SelectedIndex == -1)
                {
                    MessageBox.Show("Please select the queue to put the incoming call.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }

                sInCallType = "2";
                sInCallTo = ComboPutInQueue.SelectedItem.ToString();
            }

            return true;
        }

        private void BtnAddLine_Click(object sender, EventArgs e)
        {
            string sInCallTo = "";
            string sInCallType = "";

            if(AddLineChecks(ref sInCallTo, ref sInCallType) == false)
                return;
            
            if(ComboLine.FindStringExact(ComboLine.Text) != -1)
            {
                if(MessageBox.Show("Line already exist, do you want to update it?", "VaxTeleSIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                    return;
            }

            ArrayList objParamList = new ArrayList();

            objParamList.Add(ComboLine.Text);

            objParamList.Add(EditDisplayName.Text);
            objParamList.Add(EditUserName.Text);
            objParamList.Add(EditAuthUser.Text);
            objParamList.Add(EditPassword.Text);
            objParamList.Add(EditDomainRealm.Text);
            objParamList.Add(EditProxySIP.Text);
            objParamList.Add(EditOutboundProxySIP.Text);

            cCodec objCodec = new cCodec();
            string sCodec = "";

            for(int nCount = 0; nCount < ListBoxEnableCodec.Items.Count; nCount++)
                sCodec += objCodec.GetCodecNo((string) ListBoxEnableCodec.Items[nCount]);
            
            objParamList.Add(sCodec);

            if(CheckBoxRegLine.Checked == true)
                objParamList.Add("REGISTER-YES");
            else
                objParamList.Add("REGISTER-NO");
            
            objParamList.Add(sInCallType);
            objParamList.Add(sInCallTo);

            MainForm.m_objMainForm.SendCommand("AddLine", ref objParamList);

        }

        private void AddLineForm_Load(object sender, EventArgs e)
        {
            MainForm.m_objMainForm.SendCommand("ShowLines");
            ClearDlgContent();
        }

        private void ClearDlgContent()
        {
            ComboLine.Text = "";

            EditUserName.Text = "";
            EditAuthUser.Text = "";
            EditPassword.Text = "";
            EditDisplayName.Text = "";
            EditDomainRealm.Text = "";
            EditProxySIP.Text = "";
            EditOutboundProxySIP.Text = "";

            if(ComboConnectToUser.Items.Count > 0)
                ComboConnectToUser.SelectedIndex = 0;

            if(ComboPutInQueue.Items.Count > 0)
                ComboPutInQueue.SelectedIndex = 0;
            
            CheckBoxRegLine.Checked = false;
            RadioRejectCall.Checked = true;

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            ListBoxEnableCodec.Items.Add(cCodec.G711U_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G711A_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.ILBC_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G610_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G729_TEXT);
        }

        private void BtnRemoveLine_Click(object sender, EventArgs e)
        {
            if(ComboLine.Text.Length == 0)
            {
                MessageBox.Show("Please enter line name.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            ArrayList objParamList = new ArrayList();
            objParamList.Add(ComboLine.Text);

            MainForm.m_objMainForm.SendCommand("RemoveLine", ref objParamList);
        }

        private void UpdateDlgContent(string sLineName, string sDisplayName, string sUserName, string sAuthUser, string sAuthPwd, string sRealm, string sProxy, string sOutboundProxy, string sCodecList, string sRegister, string sInCallType, string sInCallTo)
        {
            if(ComboLine.Text != sLineName)
                return;
            
            EditDisplayName.Text = sDisplayName;
            EditUserName.Text = sUserName;

            EditAuthUser.Text = sAuthUser;
            EditPassword.Text = sAuthPwd;
            EditDomainRealm.Text = sRealm;
            
            EditProxySIP.Text = sProxy;
            EditOutboundProxySIP.Text = sOutboundProxy;

            if(sRegister == "REGISTER-YES")
                CheckBoxRegLine.Checked = true;
            else
                CheckBoxRegLine.Checked = false;
            
            UpdateIncomingCallSection(Convert.ToInt32(sInCallType), sInCallTo);

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();
                        
            cCodec objCodec = new cCodec();

            for(int nCount = 0; nCount < cCodec.TOTAL_SIZE_CODEC; nCount++)
                ListBoxDisableCodec.Items.Add(objCodec.GetCodecName(Convert.ToString(nCount)));
            
            string sCodecName;

            for (int nCount = 0; nCount < sCodecList.Length; nCount++)
            {
                sCodecName = objCodec.GetCodecName(sCodecList[nCount].ToString());
                ListBoxEnableCodec.Items.Add(sCodecName);
                ListBoxDisableCodec.Items.Remove(sCodecName);
            }
        }

        private void UpdateIncomingCallSection(int nInCallType, string sInCallTo)
        {
            if(nInCallType == IN_BOUND_CALL_REJECT)
                RadioRejectCall.Checked = true;

            else if (nInCallType == IN_BOUND_CALL_CONNECT_USER)
            {
                int nIndex = ComboConnectToUser.FindStringExact(sInCallTo);

                if (nIndex == -1)
                {
                    RadioRejectCall.Checked = true;
                    return;
                }

                if (ComboConnectToUser.Items.Count > 0)
                    ComboConnectToUser.SelectedIndex = nIndex;

                RadioConnectToUser.Checked = true;
            }
            else if (nInCallType == IN_BOUND_CALL_PUT_IN_QUEUE)
            {
                int nIndex = ComboPutInQueue.FindStringExact(sInCallTo);

                if (nIndex == -1)
                {
                    RadioRejectCall.Checked = true;
                    return;
                }

                if (ComboPutInQueue.Items.Count > 0)
                    ComboPutInQueue.SelectedIndex = nIndex;

                RadioPutInQueue.Checked = true;
            }
        }

        private void ComboLine_SelectedIndexChanged(object sender, EventArgs e)
        {
            ArrayList objParamList = new ArrayList();
            objParamList.Add(ComboLine.SelectedItem.ToString());

            MainForm.m_objMainForm.SendCommand("LineInfo", ref objParamList);
        }

        private void BtnClearLineDlg_Click(object sender, EventArgs e)
        {
            ClearDlgContent();
        }

        private void BtnDisableCodec_Click(object sender, EventArgs e)
        {
            if(ListBoxEnableCodec.SelectedItems.Count == 0) return;

            ListBoxDisableCodec.Items.Add(ListBoxEnableCodec.SelectedItem.ToString());
            ListBoxEnableCodec.Items.Remove(ListBoxEnableCodec.SelectedItem);
        }

        private void BtnEnableCodec_Click(object sender, EventArgs e)
        {
            if(ListBoxDisableCodec.SelectedItems.Count == 0) return;

            ListBoxEnableCodec.Items.Add(ListBoxDisableCodec.SelectedItem.ToString());
            ListBoxDisableCodec.Items.Remove(ListBoxDisableCodec.SelectedItem);
        }

        public void OnTelnetCommand(string sCommand, ref ArrayList objParamList)
        {
            if (String.Compare(sCommand, "AddUser", true) == 0 && objParamList.Count == 1)
            {
                ComboConnectToUser.Items.Add(objParamList[0]);
                return;
            }
        
            if(String.Compare(sCommand, "RemoveUser", true) == 0 && objParamList.Count == 1)
            {
                int nIndex = ComboConnectToUser.FindStringExact((string) objParamList[0]);
                if(nIndex == -1) return;
                
                ComboConnectToUser.Items.RemoveAt(nIndex);
                return;
            }

            if(String.Compare(sCommand, "AddQueue", true) == 0 && objParamList.Count == 1)
            {
                ComboPutInQueue.Items.Add(objParamList[0]);

                return;
            }

            if(String.Compare(sCommand, "RemoveQueue", true) == 0 && objParamList.Count == 1)
            {
                int nIndex = ComboPutInQueue.FindStringExact((string) objParamList[0]);
                if(nIndex == -1) return;
                
                ComboPutInQueue.Items.RemoveAt(nIndex);

                return;
            }

            if(String.Compare(sCommand, "ShowQueues", true) == 0)
            {
                for(int nCount = 0; nCount < objParamList.Count; nCount++)
                    ComboPutInQueue.Items.Add(objParamList[nCount]);
                
                if(ComboPutInQueue.Items.Count > 0)
                    ComboPutInQueue.SelectedIndex = 0;
                
                return;
            }

            if(String.Compare(sCommand, "ShowUsers", true) == 0)
            {
                for(int nCount = 0; nCount < objParamList.Count; nCount++)
                    ComboConnectToUser.Items.Add(objParamList[nCount]);
                
                if(ComboConnectToUser.Items.Count > 0)
                    ComboConnectToUser.SelectedIndex = 0;
                
                return;
            }

            if(String.Compare(sCommand, "ShowLines", true) == 0)
            {
                for(int nCount = 0; nCount < objParamList.Count; nCount++)
                    ComboLine.Items.Add(objParamList[nCount]);
                
                if(ComboLine.Items.Count > 0)
                    ComboLine.SelectedIndex = 0;
                
                return;
            }

            if(String.Compare(sCommand, "LineInfo", true) == 0 && objParamList.Count == 12)
            {
                string sLineName = (string) objParamList[0];
                string sDisplayName = (string) objParamList[1];
                string sUserName = (string) objParamList[2];
                string sAuthUser = (string) objParamList[3];
                string sAuthPwd = (string) objParamList[4];
                string sRealm = (string) objParamList[5];
                string sProxy = (string) objParamList[6];
                string sOutboundProxy = (string) objParamList[7];

                string sCodecList = (string) objParamList[8];

                string sRegister = (string) objParamList[9];
                string sInCallType = (string) objParamList[10];
                string sInCallTo = (string) objParamList[11];

                UpdateDlgContent(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sRealm, sProxy, sOutboundProxy, sCodecList, sRegister, sInCallType, sInCallTo);
                return;
            }

            if(String.Compare(sCommand, "RemoveLine", true) == 0 && objParamList.Count == 1)
            {
                int nIndex = ComboLine.FindStringExact((string) objParamList[0]);
                if(nIndex == -1) return;

                ComboLine.Items.RemoveAt(nIndex);

                MessageBox.Show("Line removed successfully!", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ClearDlgContent();

                return;
            }

            if(String.Compare(sCommand, "AddLine", true) == 0 && objParamList.Count == 1)
            {
                ComboLine.Items.Add(objParamList[0]);
                MessageBox.Show("Line created successfully", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return;
            }

            if(String.Compare(sCommand, "UpdateLine", true) == 0 && objParamList.Count == 1)
            {
                MessageBox.Show("Line updated successfully", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
        }
    }
}