using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace VaxTeleConfigSIP
{
    public partial class AddLoginForm : Form
    {
        public AddLoginForm()
        {
            InitializeComponent();
        }

        private Boolean AddLoginChecks()
        {

            if(ComboLogin.Text.Length == 0)
            {
                MessageBox.Show("Please enter login.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            
            string sDigit = "0123456789";

            for(int nCount = 0; nCount < ComboLogin.Text.Length; nCount++)
            {
                if(sDigit.IndexOf(ComboLogin.Text[nCount]) == -1)
                {
                    MessageBox.Show("Login should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }

            if(EditPassword.Text.Length == 0)
            {
                MessageBox.Show("Please enter login password.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ComboLogin.Text.Length > 20 || EditPassword.Text.Length > 20)
            {
                MessageBox.Show("Login && password size should not be more 20.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ComboLogin.Text.Substring(0, 2) == "00")
            {
                MessageBox.Show("Please donot enter 00 as login prefix. 00 prefix is use to prepare dial plan.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ListBoxEnableCodec.Items.Count == 0)
            {
                MessageBox.Show("Please select a codec.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            return true;

        }
        
        private void BtnAddLogin_Click(object sender, EventArgs e)
        {
            if(AddLoginChecks() == false)
                return;
        
            if(ComboLogin.FindStringExact(ComboLogin.Text) != -1)
            {
                if(MessageBox.Show("Login already exist, do you want to update it?", "VaxTeleSIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                    return;
            }

            cCodec objCodec = new cCodec();
            string sCodec = "";

            for(int nCount = 0; nCount < ListBoxEnableCodec.Items.Count; nCount++)
                sCodec += objCodec.GetCodecNo(ListBoxEnableCodec.Items[nCount].ToString());
            
            ArrayList objParamList = new ArrayList();

            objParamList.Add(ComboLogin.Text);
            objParamList.Add(EditPassword.Text);
            objParamList.Add(sCodec);

            if(CheckBoxStealthListener.Checked)
                objParamList.Add("STEALTH-YES");
            else
                objParamList.Add("STEALTH-NO");
            

            if(CheckBoxBlockAccount.Checked)
                objParamList.Add("BLOCK-YES");
            else
                objParamList.Add("BLOCK-NO");

            MainForm.m_objMainForm.SendCommand("AddUser", ref objParamList);
        }

        private void UpdateDlgContent(string sLogin, string sPassword, string sCodecList, string sStealth, string sBlock)
        {
            if(ComboLogin.Text != sLogin)
                return;
            
            EditPassword.Text = sPassword;

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            cCodec objCodec = new cCodec();

            for(int nCount = 0; nCount < cCodec.TOTAL_SIZE_CODEC; nCount++)
                ListBoxDisableCodec.Items.Add(objCodec.GetCodecName(Convert.ToString(nCount)));
            
            string sCodecName;

            for(int nCount = 0; nCount < sCodecList.Length; nCount++)
            {
                sCodecName = objCodec.GetCodecName(sCodecList[nCount].ToString());
                ListBoxEnableCodec.Items.Add(sCodecName);
                ListBoxDisableCodec.Items.Remove(sCodecName);
            }

            if(sBlock == "BLOCK-YES")
                CheckBoxBlockAccount.Checked = true;
            else
                CheckBoxBlockAccount.Checked = false;

            if(sStealth == "STEALTH-YES")
                CheckBoxStealthListener.Checked = true;
            else
                CheckBoxStealthListener.Checked = false;
        }

        private void BtnDisableCodec_Click(object sender, EventArgs e)
        {
            if(ListBoxEnableCodec.SelectedItems.Count == 0) return;

            ListBoxDisableCodec.Items.Add(ListBoxEnableCodec.SelectedItem.ToString());
            ListBoxEnableCodec.Items.Remove(ListBoxEnableCodec.SelectedItem);
        }

        private void BtnEnableCodec_Click(object sender, EventArgs e)
        {
            if(ListBoxDisableCodec.SelectedItems.Count == 0) return;

            ListBoxEnableCodec.Items.Add(ListBoxDisableCodec.SelectedItem.ToString());
            ListBoxDisableCodec.Items.Remove(ListBoxDisableCodec.SelectedItem);
        }

        private void ClearDlgContent()
        {
            ComboLogin.Text = "";
            EditPassword.Text = "";

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            CheckBoxBlockAccount.Checked = false;
            CheckBoxStealthListener.Checked = false;

            ListBoxEnableCodec.Items.Add(cCodec.G711U_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G711A_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.ILBC_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G610_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G729_TEXT);
        }

        private void AddLoginForm_Load(object sender, EventArgs e)
        {
            MainForm.m_objMainForm.SendCommand("ShowUsers");
            ClearDlgContent();
        }

        private void ComboLogin_SelectedIndexChanged(object sender, EventArgs e)
        {
            ArrayList objParamList = new ArrayList();
            objParamList.Add(ComboLogin.SelectedItem.ToString());

            MainForm.m_objMainForm.SendCommand("UserInfo", ref objParamList);
        }

        private void BtnClearLoginDlg_Click(object sender, EventArgs e)
        {
            ClearDlgContent();
        }

        private void BtnRemoveLogin_Click(object sender, EventArgs e)
        {
            if(ComboLogin.Text.Length == 0)
            {
                MessageBox.Show("Please enter login.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            ArrayList objParamList = new ArrayList();
            objParamList.Add(ComboLogin.Text);

            MainForm.m_objMainForm.SendCommand("RemoveUser", ref objParamList);
        }


        public void OnTelnetCommand(string sCommand, ref ArrayList objParamList)
        {
            if(String.Compare(sCommand, "ShowUsers", true) == 0)
            {
                for(int nCount = 0; nCount < objParamList.Count; nCount++)
                    ComboLogin.Items.Add(objParamList[nCount]);
                
                if(ComboLogin.Items.Count > 0)
                    ComboLogin.SelectedIndex = 0;
                
                return;
            }

            if(String.Compare(sCommand, "UserInfo", true) == 0 && objParamList.Count == 5)
            {
                string sLogin = (string) objParamList[0];
                string sPassword = (string) objParamList[1];
                string sCodecList = (string) objParamList[2];
                string sStealth = (string) objParamList[3];
                string sBlock = (string) objParamList[4];

                UpdateDlgContent(sLogin, sPassword, sCodecList, sStealth, sBlock);

                return;
            }

            if(String.Compare(sCommand, "RemoveUser", true) == 0 && objParamList.Count == 1)
            {
                int nIndex = ComboLogin.FindStringExact((string) objParamList[0]);
                if(nIndex == -1) return;
                
                ComboLogin.Items.RemoveAt(nIndex);

                MessageBox.Show("Login removed successfully!", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ClearDlgContent();

                return;
            }

            if(String.Compare(sCommand, "AddUser", true) == 0 && objParamList.Count == 1)
            {
                ComboLogin.Items.Add(objParamList[0]);
                MessageBox.Show("Login created successfully", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return;
            }

            if(String.Compare(sCommand, "UpdateUser", true) == 0 && objParamList.Count == 1)
            {
                MessageBox.Show("Login updated successfully", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

        }
    }
}